# Copyright 2024 Cloudification GmbH
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF
_MIN_SUPPORTED_MICROVERSION = '2.80'


class ShareBackupNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareBackupNegativeTest, cls).skip_checks()
        if not CONF.share.run_driver_assisted_backup_tests:
            raise cls.skipException("Share backup tests are disabled.")
        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    def setUp(self):
        super(ShareBackupNegativeTest, self).setUp()
        extra_specs = {
            'snapshot_support': True,
            'mount_snapshot_support': True,
        }
        share_type = self.create_share_type(extra_specs=extra_specs)
        share = self.create_share(self.shares_v2_client.share_protocol,
                                  share_type_id=share_type['id'])
        self.share_id = share["id"]
        self.backup_options = (
            CONF.share.driver_assisted_backup_test_driver_options)

    @decorators.idempotent_id('58c36c97-faf4-4fec-9a9b-7cff0d2035ab')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_create_backup_when_share_is_in_backup_creating_state(self):
        backup_name1 = data_utils.rand_name('Backup')
        backup1 = self.shares_v2_client.create_share_backup(
            self.share_id,
            name=backup_name1,
            backup_options=self.backup_options)['share_backup']

        # try create backup when share state is busy
        backup_name2 = data_utils.rand_name('Backup')
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_share_backup,
                          self.share_id,
                          name=backup_name2,
                          backup_options=self.backup_options)
        waiters.wait_for_resource_status(
            self.shares_v2_client, backup1['id'], "available",
            resource_name='share_backup')

        # delete the share backup
        self.shares_v2_client.delete_share_backup(backup1['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            backup_id=backup1['id'])

    @decorators.idempotent_id('58c36c97-faf4-4fec-9a9b-7cff0d2012ab')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_create_backup_when_share_is_in_error_state(self):
        self.admin_shares_v2_client.reset_state(self.share_id,
                                                status='error')

        # try create backup when share is not available
        backup_name = data_utils.rand_name('Backup')
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_share_backup,
                          self.share_id,
                          name=backup_name,
                          backup_options=self.backup_options)

        self.admin_shares_v2_client.reset_state(self.share_id,
                                                status='available')

    @decorators.idempotent_id('58c36c97-faf4-4fec-9a9b-7cff0d2012de')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_create_backup_when_share_has_snapshots(self):
        self.create_snapshot_wait_for_active(self.share_id,
                                             cleanup_in_class=False)

        # try create backup when share has snapshots
        backup_name = data_utils.rand_name('Backup')
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_share_backup,
                          self.share_id,
                          name=backup_name,
                          backup_options=self.backup_options)

    @decorators.idempotent_id('58c12c97-faf4-4fec-9a9b-7cff0d2012de')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_delete_backup_when_backup_is_not_available(self):
        backup = self.create_backup_wait_for_active(self.share_id)
        self.admin_shares_v2_client.reset_state_share_backup(
            backup['id'], status='creating')

        # try delete backup when share backup is not available
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.delete_share_backup,
                          backup['id'])

        self.admin_shares_v2_client.reset_state_share_backup(
            backup['id'], status='available')

    @decorators.idempotent_id('58c56c97-faf4-4fec-9a9b-7cff0d2012de')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_restore_backup_when_share_is_not_available(self):
        backup = self.create_backup_wait_for_active(self.share_id)
        self.admin_shares_v2_client.reset_state(self.share_id,
                                                status='error')

        # try restore backup when share is not available
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.restore_share_backup,
                          backup['id'])

        self.admin_shares_v2_client.reset_state(self.share_id,
                                                status='available')

    @decorators.idempotent_id('58c12998-faf4-4fec-9a9b-7cff0d2012de')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_restore_backup_when_backup_is_not_available(self):
        backup = self.create_backup_wait_for_active(self.share_id)
        self.admin_shares_v2_client.reset_state_share_backup(
            backup['id'], status='creating')

        # try restore backup when backup is not available
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.restore_share_backup,
                          backup['id'])

        self.admin_shares_v2_client.reset_state_share_backup(
            backup['id'], status='available')



# Copyright (C) 2022 China Telecom Digital Intelligence.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareTransferNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareTransferNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(
            constants.SHARE_TRANSFER_VERSION)
        if CONF.share.multitenancy_enabled:
            raise cls.skipException(
                'Only for driver_handles_share_servers = False driver mode.')

    @classmethod
    def resource_setup(cls):
        super(ShareTransferNegativeTest, cls).resource_setup()
        # create share_type with dhss=False
        extra_specs = cls.add_extra_specs_to_dict()
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

    def _create_share_transfer(self, share):
        transfer = self.shares_v2_client.create_share_transfer(
            share['id'])['transfer']
        waiters.wait_for_resource_status(
            self.shares_client, share['id'], 'awaiting_transfer')
        self.addCleanup(waiters.wait_for_resource_status, self.shares_client,
                        share['id'], 'available')
        self.addCleanup(self.shares_v2_client.delete_share_transfer,
                        transfer['id'])
        return transfer

    @decorators.idempotent_id('baf66f62-253e-40dd-a6a9-109bc7613e52')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_show_transfer_of_other_tenants(self):
        # create share
        share_name = data_utils.rand_name("tempest-share-name")
        share = self.create_share(
            name=share_name,
            share_type_id=self.share_type_id)

        # create share transfer
        transfer = self._create_share_transfer(share)

        self.assertRaises(lib_exc.NotFound,
                          self.alt_shares_v2_client.get_share_transfer,
                          transfer['id'])

    @decorators.idempotent_id('4b9e75b1-4ac6-4111-b09e-e6dacd0ac2c3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_nonexistent_transfer(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_share_transfer,
                          str(uuidutils.generate_uuid()))

    @decorators.idempotent_id('b3e26356-5eb0-4f73-b5a7-d3594cc2f30e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_transfer_of_other_tenants(self):
        # create share
        share_name = data_utils.rand_name("tempest-share-name")
        share = self.create_share(
            name=share_name,
            share_type_id=self.share_type_id)

        # create share transfer
        transfer = self._create_share_transfer(share)

        self.assertRaises(lib_exc.NotFound,
                          self.alt_shares_v2_client.delete_share_transfer,
                          transfer['id'])

    @decorators.idempotent_id('085d5971-fe6e-4497-93cb-f1eb176a10da')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_nonexistent_transfer(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_share_transfer,
                          str(uuidutils.generate_uuid()))

    @decorators.idempotent_id('cc7af032-0504-417e-8ab9-73b37bed7f85')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_accept_transfer_without_auth_key(self):
        # create share
        share_name = data_utils.rand_name("tempest-share-name")
        share = self.create_share(
            name=share_name,
            share_type_id=self.share_type_id)

        # create share transfer
        transfer = self._create_share_transfer(share)

        self.assertRaises(lib_exc.BadRequest,
                          self.alt_shares_v2_client.accept_share_transfer,
                          transfer['id'], "")

    @decorators.idempotent_id('05a6a345-7609-421f-be21-d79041970674')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_accept_transfer_with_incorrect_auth_key(self):
        # create share
        share_name = data_utils.rand_name("tempest-share-name")
        share = self.create_share(
            name=share_name,
            share_type_id=self.share_type_id)

        # create share transfer
        transfer = self._create_share_transfer(share)

        self.assertRaises(lib_exc.BadRequest,
                          self.alt_shares_v2_client.accept_share_transfer,
                          transfer['id'], "incorrect_auth_key")



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


class ExtraSpecsReadAdminTest(base.BaseSharesAdminTest):

    @classmethod
    def resource_setup(cls):
        super(ExtraSpecsReadAdminTest, cls).resource_setup()
        cls.share_type_name = data_utils.rand_name("share-type")
        cls.required_extra_specs = cls.add_extra_specs_to_dict()

        cls.share_type = cls.create_share_type(
            cls.share_type_name, extra_specs=cls.required_extra_specs)

        cls.st_id = cls.share_type["id"]
        cls.custom_extra_specs = {"key1": "value1", "key2": "value2"}
        cls.expected_extra_specs = copy.copy(cls.custom_extra_specs)
        cls.expected_extra_specs.update(cls.required_extra_specs)

        cls.shares_client.create_share_type_extra_specs(
            cls.st_id, cls.custom_extra_specs)

    @decorators.idempotent_id('d4cae104-838b-4248-8c9c-125165710786')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_one_share_type_extra_spec(self):
        es_get_one = self.shares_client.get_share_type_extra_spec(
            self.st_id, "key1")

        self.assertEqual({"key1": self.custom_extra_specs["key1"]}, es_get_one)

    @decorators.idempotent_id('508ceb85-5456-4db9-b33f-00620d7baea1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_all_share_type_extra_specs(self):
        es_get_all = self.shares_client.get_share_type_extra_specs(
            self.st_id)['extra_specs']

        self.assertEqual(self.expected_extra_specs, es_get_all)


@ddt.ddt
class ExtraSpecsWriteAdminTest(base.BaseSharesAdminTest):

    def setUp(self):
        super(ExtraSpecsWriteAdminTest, self).setUp()
        self.required_extra_specs = self.add_extra_specs_to_dict()
        self.custom_extra_specs = {"key1": "value1", "key2": "value2"}
        self.share_type_name = data_utils.rand_name("share-type")

        # Create share type
        self.share_type = self.create_share_type(
            self.share_type_name, extra_specs=self.required_extra_specs)

        self.st_id = self.share_type['id']

        # Create extra specs for share type
        self.shares_client.create_share_type_extra_specs(
            self.st_id, self.custom_extra_specs)

    @decorators.idempotent_id('baba776b-1b05-42ca-a002-bff1f8f14e13')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_one_share_type_extra_spec(self):
        self.custom_extra_specs["key1"] = "fake_value1_updated"

        # Update extra specs of share type
        update_one = self.shares_client.update_share_type_extra_spec(
            self.st_id, "key1", self.custom_extra_specs["key1"])
        self.assertEqual({"key1": self.custom_extra_specs["key1"]}, update_one)

        get = self.shares_client.get_share_type_extra_specs(
            self.st_id)['extra_specs']
        expected_extra_specs = self.custom_extra_specs
        expected_extra_specs.update(self.required_extra_specs)
        self.assertEqual(self.custom_extra_specs, get)

    @decorators.idempotent_id('374e7ff5-0d26-4778-bbb5-4ffd28521ad9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_all_share_type_extra_specs(self):
        self.custom_extra_specs["key2"] = "value2_updated"

        # Update extra specs of share type
        update_all = self.shares_client.update_share_type_extra_specs(
            self.st_id, self.custom_extra_specs)['extra_specs']
        self.assertEqual(self.custom_extra_specs, update_all)

        get = self.shares_client.get_share_type_extra_specs(
            self.st_id)['extra_specs']
        expected_extra_specs = self.custom_extra_specs
        expected_extra_specs.update(self.required_extra_specs)
        self.assertEqual(self.custom_extra_specs, get)

    @decorators.idempotent_id('129db0ce-9a4f-4a40-994f-e492174e0661')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_one_share_type_extra_spec(self):
        # Delete one extra spec for share type
        self.shares_client.delete_share_type_extra_spec(self.st_id, "key1")

        # Get metadata
        get = self.shares_client.get_share_type_extra_specs(
            self.st_id)['extra_specs']

        self.assertNotIn('key1', get)

    @decorators.idempotent_id('1b9f501d-8f34-46d0-b318-83bdfed571ec')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(*utils.deduplicate(['2.24', LATEST_MICROVERSION]))
    def test_delete_snapshot_support_extra_spec(self, version):
        """Is snapshot_support really an optional extra-spec if API > v2.24?"""
        utils.check_skip_if_microversion_not_supported(version)

        # set snapshot_support extra-spec
        self.shares_v2_client.update_share_type_extra_specs(
            self.st_id, {'snapshot_support': 'True'})

        # Get extra specs
        share_type_extra_specs = self.shares_client.get_share_type_extra_specs(
            self.st_id)['extra_specs']

        self.assertIn('snapshot_support', share_type_extra_specs)
        self.assertEqual('True', share_type_extra_specs['snapshot_support'])

        # Delete the 'snapshot_support' extra spec from the share type
        self.shares_v2_client.delete_share_type_extra_spec(
            self.st_id, 'snapshot_support', version=version)

        # Get extra specs
        share_type_extra_specs = self.shares_client.get_share_type_extra_specs(
            self.st_id)['extra_specs']

        self.assertNotIn('snapshot_support', share_type_extra_specs)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF

LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ShareTypesAdminTest(base.BaseSharesAdminTest):

    @decorators.idempotent_id('34000fb9-b595-4a10-8306-7465f9ebc45d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_share_type_create_delete(self):
        name = data_utils.rand_name("tempest-manila")
        extra_specs = self.add_extra_specs_to_dict()

        # Create share type
        st_create = self.shares_v2_client.create_share_type(
            name, extra_specs=extra_specs)
        self.assertEqual(name, st_create['share_type']['name'])
        st_id = st_create['share_type']['id']

        # Delete share type
        self.shares_v2_client.delete_share_type(st_id)

        # Verify deletion of share type
        self.shares_v2_client.wait_for_resource_deletion(st_id=st_id)
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_share_type,
                          st_id)

    def _verify_is_public_key_name(self, share_type, version):
        old_key_name = 'os-share-type-access:is_public'
        new_key_name = 'share_type_access:is_public'
        if utils.is_microversion_gt(version, "2.6"):
            self.assertIn(new_key_name, share_type)
            self.assertNotIn(old_key_name, share_type)
        else:
            self.assertIn(old_key_name, share_type)
            self.assertNotIn(new_key_name, share_type)

    def _verify_description(self, expect_des, share_type, version):
        if utils.is_microversion_ge(version, "2.41"):
            self.assertEqual(expect_des, share_type['description'])
        else:
            self.assertNotIn('description', share_type)

    @decorators.idempotent_id('228d8bab-0b31-433e-956f-9db9877e6573')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('2.0', '2.6', '2.7', '2.40', '2.41')
    def test_share_type_create_get(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        name = data_utils.rand_name("tempest-manila")
        description = None
        if utils.is_microversion_ge(version, "2.41"):
            description = "Description for share type"
        extra_specs = self.add_extra_specs_to_dict({"key": "value", })

        # Create share type
        st_create = self.create_share_type(
            name, extra_specs=extra_specs, version=version,
            description=description)
        self.assertEqual(name, st_create['name'])
        self._verify_description(
            description, st_create, version)
        self._verify_is_public_key_name(st_create, version)
        st_id = st_create["id"]

        # Get share type
        get = self.shares_v2_client.get_share_type(st_id, version=version)

        self.assertEqual(name, get["share_type"]["name"])
        self.assertEqual(st_id, get["share_type"]["id"])
        self._verify_description(description, get['share_type'], version)

        if utils.is_microversion_lt(version, "2.24"):
            # snapshot_support is an implied/required extra-spec until
            # version 2.24, and the service assumes it to be True since we
            # don't provide it during share type creation.
            extra_specs.update({"snapshot_support": 'True'})

        self.assertEqual(extra_specs, get["share_type"]["extra_specs"])
        self._verify_is_public_key_name(get['share_type'], version)

        # Check that backwards compatibility didn't break
        self.assertDictMatch(get["volume_type"], get["share_type"])

    @utils.skip_if_microversion_not_supported("2.50")
    @decorators.idempotent_id('a9af19e1-e789-4c4f-a39b-dd8df6ed00b1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.named_data(
        ('2_50_name_description_public', '2.50',
         data_utils.rand_name("type_updated"), 'description_updated', True),
        ('2_50_name', '2.50', data_utils.rand_name("type_updated"), None,
         None),
        ('2_50_description_public', '2.50', None, 'description_updated',
         None),
        ('2_50_public', '2.50', None, None, True),
        ('2_50', '2.50', None, None, False),
        (f'{LATEST_MICROVERSION}_name_description_public',
         LATEST_MICROVERSION, data_utils.rand_name("type_updated"),
         'description_updated', True),
        (f'{LATEST_MICROVERSION}_name', LATEST_MICROVERSION,
         data_utils.rand_name("type_updated"), None, None),
        (f'{LATEST_MICROVERSION}_description', LATEST_MICROVERSION, None,
         'description_updated', None),
        (f'{LATEST_MICROVERSION}_public', LATEST_MICROVERSION, None, None,
         True),
        (LATEST_MICROVERSION, LATEST_MICROVERSION, None, None, False),
    )
    def test_share_type_create_update(self, version, st_name,
                                      st_description, st_is_public):
        name = data_utils.rand_name("tempest-manila")
        description = "Description for share type"
        extra_specs = self.add_extra_specs_to_dict({"key": "value", })

        # Create share type
        st_create = self.create_share_type(
            name, extra_specs=extra_specs, version=version,
            description=description)
        self.assertEqual(name, st_create['name'])
        self._verify_description(
            description, st_create, version)
        self._verify_is_public_key_name(st_create, version)
        st_id = st_create["id"]

        # Update share type
        updated_st = self.shares_v2_client.update_share_type(
            st_id, name=st_name, is_public=st_is_public,
            description=st_description, version=version)
        if st_name is not None:
            self.assertEqual(st_name, updated_st["share_type"]["name"])
        if st_description is not None:
            self._verify_description(st_description,
                                     updated_st['share_type'], version)
        if st_is_public is not None:
            self.assertEqual(
                st_is_public,
                updated_st["share_type"]["share_type_access:is_public"])

    @utils.skip_if_microversion_not_supported("2.50")
    @decorators.idempotent_id('9019dc61-b2b1-472d-9b15-a3986439d4c3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(
        ('2.50', None, '', None),
        (LATEST_MICROVERSION, None, '', None),
    )
    @ddt.unpack
    def test_share_type_unset_description(
            self, version, st_name, st_description, st_is_public):
        name = data_utils.rand_name("tempest-manila")
        description = "Description for share type"
        extra_specs = self.add_extra_specs_to_dict({"key": "value", })

        # Create share type
        st_create = self.create_share_type(
            name, extra_specs=extra_specs, version=version,
            description=description)
        self.assertEqual(name, st_create['name'])
        self._verify_description(
            description, st_create, version)
        self._verify_is_public_key_name(st_create, version)
        st_id = st_create["id"]

        # Update share type
        updated_st = self.shares_v2_client.update_share_type(
            st_id, name=st_name, is_public=st_is_public,
            description=st_description, version=version)

        self._verify_description(None, updated_st['share_type'], version)

    @decorators.idempotent_id('5cc4c2e5-d2a4-4bfc-9208-3455ac551f20')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('2.0', '2.6', '2.7', '2.40', '2.41')
    def test_share_type_create_list(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        name = data_utils.rand_name("tempest-manila")
        description = None
        if utils.is_microversion_ge(version, "2.41"):
            description = "Description for share type"
        extra_specs = self.add_extra_specs_to_dict()

        # Create share type
        st_create = self.create_share_type(
            name, extra_specs=extra_specs, version=version,
            description=description)
        self._verify_is_public_key_name(st_create, version)
        st_id = st_create["id"]

        # list share types
        st_list = self.shares_v2_client.list_share_types(version=version)
        sts = st_list["share_types"]
        self.assertGreaterEqual(len(sts), 1)
        self.assertTrue(any(st_id in st["id"] for st in sts))
        for st in sts:
            self._verify_is_public_key_name(st, version)

        # Check that backwards compatibility didn't break
        vts = st_list["volume_types"]
        self.assertEqual(len(sts), len(vts))
        for i in range(len(sts)):
            self.assertDictMatch(sts[i], vts[i])

    @decorators.idempotent_id('4e2ad320-9f3d-4797-a3ad-bf800bcd1831')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_with_share_type(self):

        # Data
        share_name = data_utils.rand_name("share")
        shr_type_name = data_utils.rand_name("share-type")
        extra_specs = self.add_extra_specs_to_dict({
            "storage_protocol": CONF.share.capability_storage_protocol,
        })

        # Create share type
        st_create = self.create_share_type(
            shr_type_name, extra_specs=extra_specs)

        # Create share with share type
        share = self.create_share(
            name=share_name, share_type_id=st_create["id"])
        self.assertEqual(share["name"], share_name)
        waiters.wait_for_resource_status(
            self.shares_client, share["id"], "available")

        # Verify share info
        get = self.shares_v2_client.get_share(
            share["id"], version="2.5")['share']
        self.assertEqual(share_name, get["name"])
        self.assertEqual(share["id"], get["id"])
        self.assertEqual(shr_type_name, get["share_type"])

        get = self.shares_v2_client.get_share(
            share["id"], version="2.6")['share']
        self.assertEqual(st_create["id"], get["share_type"])
        self.assertEqual(shr_type_name, get["share_type_name"])

    @decorators.idempotent_id('d2261a27-d4a4-4237-9fad-f6fd8f27783a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_private_share_type_access(self):
        name = data_utils.rand_name("tempest-manila")
        extra_specs = self.add_extra_specs_to_dict({"key": "value", })
        project_id = self.shares_client.tenant_id

        # Create private share type
        st_create = self.create_share_type(
            name, False, extra_specs=extra_specs)
        self.assertEqual(name, st_create['name'])
        st_id = st_create["id"]

        # It should not be listed without access
        st_list = self.shares_v2_client.list_share_types()
        sts = st_list["share_types"]
        self.assertFalse(any(st_id in st["id"] for st in sts))

        # List projects that have access for share type - none expected
        access = self.shares_v2_client.list_access_to_share_type(
            st_id)['share_type_access']
        self.assertEmpty(access)

        # Add project access to share type
        access = self.shares_v2_client.add_access_to_share_type(
            st_id, project_id)

        # Now it should be listed
        st_list = self.shares_client.list_share_types()
        sts = st_list["share_types"]
        self.assertTrue(any(st_id in st["id"] for st in sts))

        # List projects that have access for share type - one expected
        access = self.shares_v2_client.list_access_to_share_type(
            st_id)['share_type_access']
        expected = [{'share_type_id': st_id, 'project_id': project_id}, ]
        self.assertEqual(expected, access)

        # Remove project access from share type
        access = self.shares_v2_client.remove_access_from_share_type(
            st_id, project_id)

        # It should not be listed without access
        st_list = self.shares_client.list_share_types()
        sts = st_list["share_types"]
        self.assertFalse(any(st_id in st["id"] for st in sts))

        # List projects that have access for share type - none expected
        access = self.shares_v2_client.list_access_to_share_type(
            st_id)['share_type_access']
        self.assertEmpty(access)

    @decorators.idempotent_id('90dca5c5-f28e-4f16-90ed-78f5d725664e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(*utils.deduplicate(('2.45', '2.46', LATEST_MICROVERSION)))
    def test_share_type_create_show_list_with_is_default_key(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        name = data_utils.rand_name("tempest-manila")
        extra_specs = self.add_extra_specs_to_dict()

        # Create share type
        st_create = self.create_share_type(
            name, extra_specs=extra_specs, version=version)

        if utils.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', st_create)
            self.assertIs(False, st_create['is_default'])
        else:
            self.assertNotIn('is_default', st_create)

        # list share types
        st_list = self.shares_v2_client.list_share_types(version=version)
        for st_get in st_list['share_types']:
            if utils.is_microversion_ge(version, '2.46'):
                self.assertIn('is_default', st_get)
                if st_create['id'] == st_get['id']:
                    self.assertIs(False, st_get['is_default'])
                else:
                    self.assertTrue(st_get['is_default'] in (True, False))
            else:
                self.assertNotIn('is_default', st_get)

        # show share types
        st_id = st_create['id']
        st_show = self.shares_v2_client.get_share_type(
            st_id, version=version)['share_type']

        if utils.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', st_show)
            self.assertIs(False, st_show['is_default'])
        else:
            self.assertNotIn('is_default', st_show)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF

LOCKS_MIN_API_VERSION = '2.81'


class ResourceLockNegativeTestAPIOnly(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ResourceLockNegativeTestAPIOnly, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(LOCKS_MIN_API_VERSION)

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @decorators.idempotent_id('dd978cf7-1622-49e8-a6c8-3da4ac6c6f86')
    def test_create_resource_lock_invalid_resource(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.create_resource_lock,
            'invalid-share-id',
            'share'
        )

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @decorators.idempotent_id('d5600bdc-72c8-43fd-9900-c112aa6c87fa')
    def test_delete_resource_lock_invalid(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.delete_resource_lock,
            'invalid-lock-id'
        )


class ResourceLockNegativeTestWithShares(base.BaseSharesMixedTest):
    @classmethod
    def skip_checks(cls):
        super(ResourceLockNegativeTestWithShares, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(LOCKS_MIN_API_VERSION)

    @classmethod
    def resource_setup(cls):
        super(ResourceLockNegativeTestWithShares, cls).resource_setup()
        share_type = cls.create_share_type()
        cls.share = cls.create_share(share_type_id=share_type['id'])
        cls.user_project = cls.os_admin.projects_client.show_project(
            cls.shares_v2_client.project_id)['project']

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('658297a8-d675-471d-8a19-3d9e9af3a352')
    def test_create_resource_lock_invalid_resource_action(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.create_resource_lock,
            self.share['id'],
            'share',
            resource_action='invalid-action'
        )

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('0057b3e7-c250-492d-805b-e355dff954ed')
    def test_create_resource_lock_invalid_lock_reason_too_long(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.create_resource_lock,
            self.share['id'],
            'share',
            resource_action='delete',
            lock_reason='invalid' * 150,
        )

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('a2db3d29-b42f-4c0b-b484-afd32f91f747')
    def test_update_resource_lock_invalid_param(self):
        lock = self.create_resource_lock(self.share['id'])
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.update_resource_lock,
            lock['id'],
            resource_action='invalid-action'
        )
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.update_resource_lock,
            lock['id'],
            lock_reason='invalid' * 150,
        )

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('45b12120-0fc3-461f-8776-fdb92e599394')
    def test_update_resource_lock_created_by_different_user(self):
        lock = self.create_resource_lock(self.share['id'])
        new_user = self.create_user_and_get_client(project=self.user_project)
        self.assertRaises(
            lib_exc.Forbidden,
            new_user.shares_v2_client.update_resource_lock,
            lock['id'],
            lock_reason="I shouldn't be able to do this",
        )

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('00a8ef2b-8769-4aad-aefc-43fc579492f7')
    def test_delete_resource_lock_created_by_different_user(self):
        lock = self.create_resource_lock(self.share['id'])
        new_user = self.create_user_and_get_client(project=self.user_project)
        self.assertRaises(
            lib_exc.Forbidden,
            new_user.shares_v2_client.delete_resource_lock,
            lock['id'],
        )



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.api import test_share_networks
from manila_tempest_tests import utils


class ShareNetworkAdminTest(base.BaseSharesMixedTest,
                            test_share_networks.ShareNetworkListMixin):

    @classmethod
    def resource_setup(cls):
        super(ShareNetworkAdminTest, cls).resource_setup()
        ss_data = utils.generate_security_service_data()
        cls.ss_ldap = cls.create_security_service(**ss_data)

        cls.data_sn_with_ldap_ss = {
            'name': 'sn_with_ldap_ss',
            'neutron_net_id': '1111',
            'neutron_subnet_id': '2222',
            'created_at': '2002-02-02',
            'updated_at': None,
            'network_type': 'vlan',
            'segmentation_id': 1000,
            'cidr': '10.0.0.0/24',
            'ip_version': 4,
            'description': 'fake description',
        }
        cls.sn_with_ldap_ss = cls.create_share_network(
            cleanup_in_class=True,
            add_security_services=False,
            **cls.data_sn_with_ldap_ss)

        cls.shares_client.add_sec_service_to_share_network(
            cls.sn_with_ldap_ss["id"],
            cls.ss_ldap["id"])

        cls.data_sn_with_kerberos_ss = {
            'name': 'sn_with_kerberos_ss',
            'created_at': '2003-03-03',
            'updated_at': None,
            'neutron_net_id': 'test net id',
            'neutron_subnet_id': 'test subnet id',
            'network_type': 'local',
            'segmentation_id': 2000,
            'cidr': '10.0.0.0/13',
            'ip_version': 6,
            'description': 'fake description',
        }

        cls.ss_kerberos = cls.alt_shares_v2_client.create_security_service(
            ss_type='kerberos',
            **cls.data_sn_with_ldap_ss)['security_service']

        cls.sn_with_kerberos_ss = (
            cls.alt_shares_v2_client.create_share_network(
                cleanup_in_class=True,
                add_security_services=False,
                **cls.data_sn_with_kerberos_ss)['share_network']
        )

        cls.alt_shares_v2_client.add_sec_service_to_share_network(
            cls.sn_with_kerberos_ss["id"],
            cls.ss_kerberos["id"])

    @decorators.idempotent_id('983fb22d-3057-402f-8988-62ce41a557fb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_networks_all_tenants(self):
        listed = self.admin_shares_v2_client.list_share_networks_with_detail(
            {'all_tenants': 1})['share_networks']
        self.assertTrue(any(self.sn_with_ldap_ss['id'] == sn['id']
                            for sn in listed))
        self.assertTrue(any(self.sn_with_kerberos_ss['id'] == sn['id']
                            for sn in listed))

    @decorators.idempotent_id('36c26b6b-8984-4255-959b-74f6ef46c37b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_networks_filter_by_project_id(self):
        listed = self.admin_shares_v2_client.list_share_networks_with_detail(
            {
                'project_id': self.sn_with_kerberos_ss['project_id']
            })['share_networks']
        self.assertTrue(any(self.sn_with_kerberos_ss['id'] == sn['id']
                            for sn in listed))
        self.assertTrue(all(self.sn_with_kerberos_ss['project_id'] ==
                            sn['project_id'] for sn in listed))



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_log import log
from tempest import config
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion
LOG = log.getLogger(__name__)


@ddt.ddt
class SecurityServiceListMixin(object):

    @decorators.idempotent_id('f6f5657c-a93c-49ed-86e3-b351a92734d5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_security_services(self):
        listed = self.shares_client.list_security_services(
            )['security_services']
        self.assertTrue(any(self.ss_ldap['id'] == ss['id'] for ss in listed))
        self.assertTrue(any(self.ss_kerberos['id'] == ss['id']
                            for ss in listed))

        # verify keys
        keys = ["name", "id", "status", "type", ]
        [self.assertIn(key, s_s.keys()) for s_s in listed for key in keys]

    @decorators.idempotent_id('22b22937-7436-458c-ac22-8ff19feab253')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(*utils.deduplicate(['1.0', '2.42', '2.44', LATEST_MICROVERSION]))
    def test_list_security_services_with_detail(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        with_ou = True if utils.is_microversion_ge(version, '2.44') else False
        if utils.is_microversion_ge(version, '2.0'):
            listed = self.shares_v2_client.list_security_services(
                detailed=True, version=version)['security_services']
        else:
            listed = self.shares_client.list_security_services(
                detailed=True)['security_services']

        self.assertTrue(any(self.ss_ldap['id'] == ss['id'] for ss in listed))
        self.assertTrue(any(self.ss_kerberos['id'] == ss['id']
                            for ss in listed))

        # verify keys
        keys = [
            "name", "id", "status", "description",
            "domain", "server", "dns_ip", "user", "password", "type",
            "created_at", "updated_at", "project_id",
        ]
        [self.assertIn(key, s_s.keys()) for s_s in listed for key in keys]

        for ss in listed:
            self.assertEqual(with_ou, 'ou' in ss.keys())

    @decorators.idempotent_id('88f62835-0aee-4bed-a37f-ffd99430da8a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_list_security_services_filter_by_share_network(self):
        sn = self.shares_client.get_share_network(
            self.shares_client.share_network_id)['share_network']
        fresh_sn = []
        for i in range(2):
            sn = self.create_share_network(
                add_security_services=False,
                neutron_net_id=sn["neutron_net_id"],
                neutron_subnet_id=sn["neutron_subnet_id"])
            fresh_sn.append(sn)

        self.shares_client.add_sec_service_to_share_network(
            fresh_sn[0]["id"], self.ss_ldap["id"])
        self.shares_client.add_sec_service_to_share_network(
            fresh_sn[1]["id"], self.ss_kerberos["id"])

        listed = self.shares_client.list_security_services(
            params={
                'share_network_id': fresh_sn[0]['id']
            })['security_services']
        self.assertEqual(1, len(listed))
        self.assertEqual(self.ss_ldap['id'], listed[0]['id'])

        keys = ["name", "id", "status", "type", ]
        [self.assertIn(key, s_s.keys()) for s_s in listed for key in keys]

    @decorators.idempotent_id('f055faad-dd36-4eed-9b50-61280931dea2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_security_services_detailed_filter_by_ss_attributes(self):
        search_opts = {
            'name': 'ss_ldap',
            'type': 'ldap',
            'user': 'fake_user',
            'server': 'fake_server_1',
            'dns_ip': '1.1.1.1',
            'domain': 'fake_domain_1',
        }
        listed = self.shares_client.list_security_services(
            detailed=True,
            params=search_opts)['security_services']
        self.assertTrue(any(self.ss_ldap['id'] == ss['id'] for ss in listed))
        for ss in listed:
            self.assertTrue(all(ss[key] == value for key, value
                                in search_opts.items()))


@ddt.ddt
class SecurityServicesTest(base.BaseSharesMixedTest,
                           SecurityServiceListMixin):

    @classmethod
    def resource_setup(cls):
        super(SecurityServicesTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

    def setUp(self):
        super(SecurityServicesTest, self).setUp()
        ss_ldap_data = {
            'name': 'ss_ldap',
            'dns_ip': '1.1.1.1',
            'server': 'fake_server_1',
            'domain': 'fake_domain_1',
            'user': 'fake_user',
            'password': 'pass',
        }
        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.44'):
            ss_ldap_data['ou'] = 'OU=fake_unit_1'
        ss_kerberos_data = {
            'name': 'ss_kerberos',
            'dns_ip': '2.2.2.2',
            'server': 'fake_server_2',
            'domain': 'fake_domain_2',
            'user': 'test_user',
            'password': 'word',
        }
        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.44'):
            ss_kerberos_data['ou'] = 'OU=fake_unit_2'
        self.ss_ldap = self.create_security_service('ldap', **ss_ldap_data)
        self.ss_kerberos = self.create_security_service(
            'kerberos', **ss_kerberos_data)

    @decorators.idempotent_id('70927e29-4a6a-431a-bbc1-76bc419e0579')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_delete_security_service(self):
        data = utils.generate_security_service_data()
        self.service_names = ["ldap", "kerberos", "active_directory"]
        for ss_name in self.service_names:
            ss = self.create_security_service(ss_name, **data)
            self.assertDictContainsSubset(data, ss)
            self.assertEqual(ss_name, ss["type"])
            self.shares_client.delete_security_service(ss["id"])

    @decorators.idempotent_id('bb052be4-0176-4613-b7d5-e12bef391ddb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(*utils.deduplicate(['1.0', '2.43', '2.44', LATEST_MICROVERSION]))
    def test_get_security_service(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        with_ou = True if utils.is_microversion_ge(version, '2.44') else False
        data = utils.generate_security_service_data(set_ou=with_ou)

        if utils.is_microversion_ge(version, '2.0'):
            ss = self.create_security_service(
                client=self.shares_v2_client, version=version, **data)
            get = self.shares_v2_client.get_security_service(
                ss["id"], version=version)['security_service']
        else:
            ss = self.create_security_service(**data)
            get = self.shares_client.get_security_service(
                ss["id"])['security_service']

        self.assertDictContainsSubset(data, ss)
        self.assertEqual(with_ou, 'ou' in ss)
        self.assertDictContainsSubset(data, get)
        self.assertEqual(with_ou, 'ou' in get)

    @decorators.idempotent_id('84d47747-13c8-4ab9-9fc4-a43fbb29ad18')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_security_service(self):
        data = utils.generate_security_service_data()
        ss = self.create_security_service(**data)
        self.assertDictContainsSubset(data, ss)

        upd_data = utils.generate_security_service_data()
        updated = self.shares_client.update_security_service(
            ss["id"], **upd_data)['security_service']

        get = self.shares_client.get_security_service(
            ss["id"])['security_service']
        self.assertDictContainsSubset(upd_data, updated)
        self.assertDictContainsSubset(upd_data, get)

        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.44'):
            # update again with ou
            upd_data_ou = utils.generate_security_service_data(set_ou=True)
            updated_ou = self.shares_v2_client.update_security_service(
                ss["id"], **upd_data_ou)['security_service']

            get_ou = self.shares_v2_client.get_security_service(
                ss["id"])['security_service']
            self.assertDictContainsSubset(upd_data_ou, updated_ou)
            self.assertDictContainsSubset(upd_data_ou, get_ou)

    @decorators.idempotent_id('c3c04992-da11-4677-9098-eff3f4231a4b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_try_update_valid_keys_sh_server_exists(self):
        ss_data = utils.generate_security_service_data()
        ss = self.create_security_service(**ss_data)

        sn = self.shares_client.get_share_network(
            self.shares_client.share_network_id)['share_network']
        fresh_sn = self.create_share_network(
            add_security_services=False,
            neutron_net_id=sn["neutron_net_id"],
            neutron_subnet_id=sn["neutron_subnet_id"])

        self.shares_client.add_sec_service_to_share_network(
            fresh_sn["id"], ss["id"])

        # Security service with fake data is used, so if we use backend driver
        # that fails on wrong data, we expect error here.
        # We require any share that uses our share-network.
        try:
            self.create_share(share_type_id=self.share_type_id,
                              share_network_id=fresh_sn["id"],
                              cleanup_in_class=False)
        except Exception as e:
            # we do wait for either 'error' or 'available' status because
            # it is the only available statuses for proper deletion.
            LOG.warning("Caught exception. It is expected in case backend "
                        "fails having security-service with improper data "
                        "that leads to share-server creation error. "
                        "%s", str(e))

        update_data = {
            "name": "name",
            "description": "new_description",
        }
        updated = self.shares_client.update_security_service(
            ss["id"], **update_data)['security_service']
        self.assertDictContainsSubset(update_data, updated)

    @decorators.idempotent_id('8d9af272-df89-470d-9ff8-92ba774c9fff')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_security_services_filter_by_invalid_opt(self):
        listed = self.shares_client.list_security_services(
            params={'fake_opt': 'some_value'})['security_services']
        self.assertTrue(any(self.ss_ldap['id'] == ss['id'] for ss in listed))
        self.assertTrue(any(self.ss_kerberos['id'] == ss['id']
                            for ss in listed))

    @decorators.idempotent_id('d501710e-4710-4c13-a373-75ed6ababb13')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_try_list_security_services_all_tenants_ignored(self):
        alt_security_service = self.create_security_service(
            **utils.generate_security_service_data(),
            client=self.alt_shares_v2_client)
        alt_security_service_id = alt_security_service['id']
        sec_service_list = self.shares_client.list_security_services(
            params={'all_tenants': 1})['security_services']
        sec_service_ids = [ss['id'] for ss in sec_service_list]
        self.assertTrue(
            any(self.ss_ldap['id'] == ss['id'] for ss in sec_service_list))
        self.assertTrue(
            any(self.ss_kerberos['id'] == ss['id'] for ss in sec_service_list))
        self.assertNotIn(alt_security_service_id, sec_service_ids)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacUserMessageTests(rbac_base.ShareRbacBaseTests,
                                metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacUserMessageTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.share_admin_client = cls.os_project_admin.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        # One of the options for generating a user message is to create a share
        # type with invalid extra_specs. Creating a manila share with this
        # share type will fail because no valid host is found.
        extra_specs = {
            'key': 'value',
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        share_type_name = data_utils.rand_name('share-type')
        cls.share_type = cls.share_admin_client.create_share_type(
            name=share_type_name, extra_specs=extra_specs)['share_type']
        cls.addClassResourceCleanup(
            cls.share_admin_client.delete_share_type, cls.share_type['id'])

    def create_user_message(self, client, cleanup=True):
        # Trigger a 'no valid host' situation to generate a message.
        share = client.create_share(
            share_type_id=self.share_type['id'])['share']
        self.addCleanup(client.delete_share, share['id'])
        waiters.wait_for_resource_status(client, share['id'], 'error')

        message = waiters.wait_for_message(client, share['id'])
        if cleanup:
            self.addCleanup(client.delete_message, message['id'])
        return message

    @abc.abstractmethod
    def test_list_messages(self):
        pass

    @abc.abstractmethod
    def test_show_message(self):
        pass

    @abc.abstractmethod
    def test_delete_message(self):
        pass


class ProjectAdminTests(ShareRbacUserMessageTests, base.BaseSharesTest):

    credentials = ['project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(ProjectAdminTests, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('2067d8ba-953d-4035-b65d-6001b3d4ea8f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_messages(self):
        message = self.create_user_message(
            self.share_member_client, self.share_type)
        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, self.share_type)

        message_list = self.do_request(
            'list_messages', expected_status=200)['messages']
        message_id_list = [
            s['id'] for s in message_list
        ]

        self.assertIn(message['id'], message_id_list)
        self.assertIn(message_alt['id'], message_id_list)

    @decorators.idempotent_id('ec46f10e-c768-4df5-b75a-0ce3e22d8038')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_message(self):
        message = self.create_user_message(
            self.share_member_client, self.share_type)
        self.do_request(
            'get_message', message_id=message['id'], expected_status=200)

        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, self.share_type)
        self.do_request(
            'get_message', message_id=message_alt['id'], expected_status=200)

    @decorators.idempotent_id('b91c355b-a5f8-47aa-8ab4-00a350f8ac7f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_message(self):
        message = self.create_user_message(
            self.share_member_client, cleanup=False)
        self.do_request(
            'delete_message', message_id=message['id'], expected_status=204)

        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, cleanup=False)
        self.do_request(
            'delete_message', message_id=message_alt['id'],
            expected_status=204)


class ProjectMemberTests(ShareRbacUserMessageTests, base.BaseSharesTest):
    """Test suite for basic share use message operations by member user

    In order to test share user message operations we need to preform an action
    that generates a user message. One of the reasons for generating a user
    message is share creation that fails because no valid host is found.
    To achieve this goal we need to create a share type.
    Since only user with admin credentials can create a share type, we have to
    initialize these credentials within project member class.
    """

    credentials = ['project_member', 'project_admin', 'project_alt_member']

    @decorators.idempotent_id('1fd0f86d-cb1e-4694-a54e-4b7774c7c652')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_messages(self):
        share_client = getattr(self, 'share_member_client', self.client)
        message = self.create_user_message(share_client, self.share_type)

        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, self.share_type)

        message_list = self.do_request(
            'list_messages', expected_status=200)['messages']
        message_id_list = [
            s['id'] for s in message_list
        ]

        self.assertIn(message['id'], message_id_list)
        self.assertNotIn(message_alt['id'], message_id_list)

    @decorators.idempotent_id('283d33be-727b-4180-a503-95d31cc99a79')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_message(self):
        share_client = getattr(self, 'share_member_client', self.client)
        message = self.create_user_message(share_client, self.share_type)
        self.do_request(
            'get_message', message_id=message['id'], expected_status=200)

        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, self.share_type)
        self.do_request(
            'get_message', message_id=message_alt['id'],
            expected_status=lib_exc.NotFound)

    @decorators.idempotent_id('5821a3a9-6194-414a-9668-0d933a0d4fb0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_message(self):
        share_client = getattr(self, 'share_member_client', self.client)
        message = self.create_user_message(
            share_client, cleanup=False)
        self.do_request(
            'delete_message', message_id=message['id'],
            expected_status=204)

        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, cleanup=False)
        self.do_request(
            'delete_message', message_id=message_alt['id'],
            expected_status=lib_exc.NotFound)
        self.addCleanup(self.share_admin_client.delete_message,
                        message_alt['id'])


class ProjectReaderTests(ProjectMemberTests):
    """Test suite for basic share use message operations by reader user

    In order to test certain share operations we must create a share resource
    for this. Since reader user is limited in resources creation, we are forced
    to use admin credentials, so we can test other share operations.
    In this class we use admin user to create a member user within reader
    project. That way we can perform a reader actions on this resource.
    """

    credentials = ['project_reader', 'project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(ProjectReaderTests, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('ab3b8812-47df-4472-a410-7f84d52999f3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_messages(self):
        super(ProjectReaderTests, self).test_list_messages()

    @decorators.idempotent_id('f0603a61-b620-4f89-afc5-006d1195fa7f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_message(self):
        super(ProjectReaderTests, self).test_show_message()

    @decorators.idempotent_id('a03695c7-e05a-4c89-9a04-7d94a8dd2419')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_message(self):
        message = self.create_user_message(
            self.share_member_client, cleanup=False)
        self.do_request(
            'delete_message', message_id=message['id'],
            expected_status=lib_exc.Forbidden)
        self.addCleanup(self.share_admin_client.delete_message, message['id'])

        message_alt = self.create_user_message(
            self.alt_project_share_v2_client, cleanup=False)
        self.do_request(
            'delete_message', message_id=message_alt['id'],
            expected_status=lib_exc.Forbidden)
        self.addCleanup(self.share_admin_client.delete_message,
                        message_alt['id'])



# Copyright 2018 Huawei Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class AccessRulesMetadataTest(base.BaseSharesMixedTest):
    """A Test class to test access rule metadata generically.

    Tests in this class don't care about the type of access rule or the
    protocol of the share created. They are meant to test the API semantics
    of the access rule metadata APIs.
    """

    @classmethod
    def skip_checks(cls):
        super(AccessRulesMetadataTest, cls).skip_checks()
        if not (any(p in CONF.share.enable_ip_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_user_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_cert_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_cephx_rules_for_protocols
                    for p in cls.protocols)):
            cls.message = "Rule tests are disabled"
            raise cls.skipException(cls.message)

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_ACCESS_METADATA_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(AccessRulesMetadataTest, cls).resource_setup()
        cls.access_type, __ = utils.get_access_rule_data_from_config(
            cls.shares_v2_client.share_protocol)
        int_range = range(20, 50)
        cls.access_to = {
            # list of unique values is required for ability to create lots
            # of access rules for one share using different API microversions.
            'ip': set([utils.rand_ipv6_ip() for i in int_range]),
            # following users are fakes and access rules that use it are
            # expected to fail, but they are used only for API testing.
            'user': ['foo_user_%d' % i for i in int_range],
            'cert': ['tenant_%d.example.com' % i for i in int_range],
            'cephx': ['eve%d' % i for i in int_range],
        }
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)
        cls.md1 = {"key1": "value1", "key2": "value2"}
        cls.access = cls.shares_v2_client.create_access_rule(
            cls.share["id"], cls.access_type,
            cls.access_to[cls.access_type].pop(), 'rw',
            metadata=cls.md1)['access']
        waiters.wait_for_resource_status(
            cls.shares_v2_client, cls.share["id"], "active",
            resource_name='access_rule', rule_id=cls.access["id"])

    @decorators.idempotent_id('4c8e0236-2e7b-4337-be3c-17b51a738644')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_get_delete_access_metadata(self):
        data = {"key1": "v" * 255, "k" * 255: "value2"}
        # set metadata
        access = self.allow_access(
            self.share["id"], access_type=self.access_type,
            access_to=self.access_to[self.access_type].pop(),
            access_level='rw', metadata=data)

        # read metadata
        get_access = self.shares_v2_client.get_access_rule(
            access["id"])['access']

        # verify metadata
        self.assertEqual(data, get_access['metadata'])

        # delete metadata
        for key in data.keys():
            self.shares_v2_client.delete_access_metadata(access["id"], key)

        # verify deletion of metadata
        access_without_md = self.shares_v2_client.get_access_rule(
            access["id"])['access']
        self.assertEqual({}, access_without_md['metadata'])

    @decorators.idempotent_id('8c294d7d-0702-49ce-b964-0945ec323370')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_update_metadata_by_key(self):
        md2 = {"key7": "value7", "key2": "value6_new"}

        # update metadata
        self.shares_v2_client.update_access_metadata(
            access_id=self.access['id'], metadata=md2)
        # get metadata
        get_access = self.shares_v2_client.get_access_rule(
            self.access['id'])['access']

        # verify metadata
        self.md1.update(md2)
        self.assertEqual(self.md1, get_access['metadata'])

    @decorators.idempotent_id('c97cc976-2c2f-4db3-ba07-cd04af29c9f4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_access_filter_by_metadata(self):
        data = {"key3": "v3", "key4": "value4"}
        # set metadata
        access = self.allow_access(
            self.share["id"], access_type=self.access_type,
            access_to=self.access_to[self.access_type].pop(),
            access_level='rw', metadata=data)

        # list metadata with metadata filter
        list_access = self.shares_v2_client.list_access_rules(
            share_id=self.share["id"],
            metadata={'metadata': data})['access_list']

        # verify metadata
        self.assertEqual(1, len(list_access))
        self.assertEqual(access['metadata'], list_access[0]['metadata'])
        self.assertEqual(access['id'], list_access[0]['id'])



# Copyright 2015 Hitachi Data Systems.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class MigrationBase(base.BaseSharesAdminTest):
    """Base test class for Share Migration.

    Tests share migration in multi-backend environment.

    This class covers:
    1) Driver-assisted migration: force_host_assisted_migration, nondisruptive,
    writable and preserve-metadata are False.
    2) Host-assisted migration: force_host_assisted_migration is True,
    nondisruptive, writable, preserve-metadata and preserve-snapshots are
    False.
    3) 2-phase migration of both Host-assisted and Driver-assisted.
    4) Cancelling migration past first phase.
    5) Changing driver modes through migration.

    No need to test with writable, preserve-metadata and non-disruptive as
    True, values are supplied to the driver which decides what to do. Test
    should be positive, so not being writable, not preserving metadata and
    being disruptive is less restrictive for drivers, which would abort if they
    cannot handle them.

    Drivers that implement driver-assisted migration should enable the
    configuration flag to be tested.
    """

    protocol = None

    @classmethod
    def resource_setup(cls):
        super(MigrationBase, cls).resource_setup()
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled." % cls.protocol
            raise cls.skipException(message)
        if not (CONF.share.run_host_assisted_migration_tests or
                CONF.share.run_driver_assisted_migration_tests):
            raise cls.skipException("Share migration tests are disabled.")
        cls.pools = cls.shares_v2_client.list_pools(detail=True)['pools']

        if len(cls.pools) < 2:
            raise cls.skipException("At least two different pool entries are "
                                    "needed to run share migration tests.")

        # create share type (generic)
        cls.share_type = cls.create_share_type(
            name=data_utils.rand_name('original_share_type_for_migration'),
            cleanup_in_class=True,
            extra_specs=utils.get_configured_extra_specs())
        cls.share_type_id = cls.share_type['id']

        cls.new_type = cls.create_share_type(
            name=data_utils.rand_name('new_share_type_for_migration'),
            cleanup_in_class=True,
            extra_specs=utils.get_configured_extra_specs())

        cls.new_type_opposite = cls.create_share_type(
            name=data_utils.rand_name('new_share_type_for_migration_opposite'),
            cleanup_in_class=True,
            extra_specs=utils.get_configured_extra_specs(
                variation='opposite_driver_modes'))

    def _setup_migration(self, share, opposite=False):

        if opposite:
            dest_type = self.new_type_opposite
        else:
            dest_type = self.new_type

        dest_pool = utils.choose_matching_backend(share, self.pools, dest_type)

        if opposite:
            if not dest_pool:
                raise self.skipException(
                    "This test requires two pools enabled with different "
                    "driver modes.")
        else:
            self.assertIsNotNone(dest_pool)
            self.assertIsNotNone(dest_pool.get('name'))

        old_exports = self.shares_v2_client.list_share_export_locations(
            share['id'])['export_locations']
        self.assertNotEmpty(old_exports)
        old_exports = [x['path'] for x in old_exports
                       if x['is_admin_only'] is False]
        self.assertNotEmpty(old_exports)

        self.shares_v2_client.create_access_rule(
            share['id'], access_to="50.50.50.50", access_level="rw")

        waiters.wait_for_resource_status(
            self.shares_v2_client,
            share['id'], constants.RULE_STATE_ACTIVE,
            status_attr='access_rules_status')

        self.shares_v2_client.create_access_rule(
            share['id'], access_to="51.51.51.51", access_level="ro")

        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], constants.RULE_STATE_ACTIVE,
            status_attr='access_rules_status')

        dest_pool = dest_pool['name']
        share = self.shares_v2_client.get_share(share['id'])['share']

        return share, dest_pool

    def _validate_migration_successful(self, dest_pool, share, status_to_wait,
                                       version=CONF.share.max_api_microversion,
                                       complete=True, share_network_id=None,
                                       share_type_id=None):

        statuses = ((status_to_wait,)
                    if not isinstance(status_to_wait, (tuple, list, set))
                    else status_to_wait)

        new_exports = self.shares_v2_client.list_share_export_locations(
            share['id'], version=version)['export_locations']
        self.assertNotEmpty(new_exports)
        new_exports = [x['path'] for x in new_exports if
                       x['is_admin_only'] is False]
        self.assertNotEmpty(new_exports)

        self.assertIn(share['task_state'], statuses)
        if share_network_id:
            self.assertEqual(share_network_id, share['share_network_id'])
        if share_type_id:
            self.assertEqual(share_type_id, share['share_type'])

        # Share migrated
        if complete:
            self.assertEqual(dest_pool, share['host'])

            rules = self.shares_v2_client.list_access_rules(
                share['id'])['access_list']
            expected_rules = [{
                'state': constants.RULE_STATE_ACTIVE,
                'access_to': '50.50.50.50',
                'access_type': 'ip',
                'access_level': 'rw',
            }, {
                'state': constants.RULE_STATE_ACTIVE,
                'access_to': '51.51.51.51',
                'access_type': 'ip',
                'access_level': 'ro',
            }]
            filtered_rules = [{'state': rule['state'],
                               'access_to': rule['access_to'],
                               'access_level': rule['access_level'],
                               'access_type': rule['access_type']}
                              for rule in rules]

            for r in expected_rules:
                self.assertIn(r, filtered_rules)
            self.assertEqual(len(expected_rules), len(filtered_rules))

            # In v 2.54 and beyond, we expect key 'progress' in the destination
            # share data
            if utils.is_microversion_supported('2.54'):
                self.assertEqual('100%', share['progress'])

        # Share not migrated yet
        else:
            self.assertNotEqual(dest_pool, share['host'])

    def _check_migration_enabled(self, force_host_assisted):

        if force_host_assisted:
            if not CONF.share.run_host_assisted_migration_tests:
                raise self.skipException(
                    "Host-assisted migration tests are disabled.")
        else:
            if not CONF.share.run_driver_assisted_migration_tests:
                raise self.skipException(
                    "Driver-assisted migration tests are disabled.")

    def _create_secondary_share_network(self, old_share_network_id):

        old_share_network = self.shares_v2_client.get_share_network(
            old_share_network_id)['share_network']
        share_net_info = (
            utils.share_network_get_default_subnet(old_share_network)
            if utils.share_network_subnets_are_supported()
            else old_share_network)
        new_share_network = self.create_share_network(
            cleanup_in_class=True,
            neutron_net_id=share_net_info['neutron_net_id'],
            neutron_subnet_id=share_net_info['neutron_subnet_id'])

        return new_share_network['id']

    def _test_resize_post_migration(self, force_host_assisted, resize):
        self._check_migration_enabled(force_host_assisted)
        new_size = CONF.share.share_size + 1
        share = self.create_share(self.protocol,
                                  size=new_size,
                                  share_type_id=self.share_type_id)
        share = self.shares_v2_client.get_share(share['id'])['share']

        share, dest_pool = self._setup_migration(share)

        task_state, new_share_network_id, new_share_type_id = (
            self._get_migration_data(share, force_host_assisted))

        share = self.migrate_share(
            share['id'], dest_pool,
            force_host_assisted_migration=force_host_assisted,
            wait_for_status=task_state, new_share_type_id=new_share_type_id,
            new_share_network_id=new_share_network_id)

        share = self.migration_complete(share['id'], dest_pool)
        if resize == 'extend':
            new_size = CONF.share.share_size + 2
            self.shares_v2_client.extend_share(share['id'], new_size)
            waiters.wait_for_resource_status(
                self.shares_v2_client, share['id'], constants.STATUS_AVAILABLE)
            share = self.shares_v2_client.get_share(share["id"])['share']
            self.assertEqual(new_size, int(share["size"]))
        else:
            new_size = CONF.share.share_size
            self.shares_v2_client.shrink_share(share['id'], new_size)
            waiters.wait_for_resource_status(
                self.shares_v2_client, share['id'], constants.STATUS_AVAILABLE)
            share = self.shares_v2_client.get_share(share["id"])['share']
            self.assertEqual(new_size, int(share["size"]))

        self._cleanup_share(share)

    def _get_migration_data(self, share, force_host_assisted=False):
        task_state = (constants.TASK_STATE_DATA_COPYING_COMPLETED
                      if force_host_assisted
                      else constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE)

        old_share_network_id = share['share_network_id']

        if CONF.share.multitenancy_enabled:
            new_share_network_id = self._create_secondary_share_network(
                old_share_network_id)

        else:
            new_share_network_id = None

        new_share_type_id = self.new_type['id']
        return task_state, new_share_network_id, new_share_type_id

    def _validate_snapshot(self, share, snapshot1, snapshot2):
        snapshot_list = self.shares_v2_client.list_snapshots_for_share(
            share['id'])['snapshots']
        msg = "Share %s has no snapshot." % share['id']
        # Verify that snapshot list is not empty
        self.assertNotEmpty(snapshot_list, msg)
        snapshot_id_list = [snap['id'] for snap in snapshot_list]

        # Verify that after migration original snapshots are retained
        self.assertIn(snapshot1['id'], snapshot_id_list)
        self.assertIn(snapshot2['id'], snapshot_id_list)
        # Verify that a share can be created from a snapshot after migration
        snapshot1_share = self.create_share(
            self.protocol,
            size=share['size'],
            snapshot_id=snapshot1['id'],
            share_network_id=share['share_network_id'])
        self.assertEqual(snapshot1['id'], snapshot1_share['snapshot_id'])
        self._cleanup_share(share)

    def _validate_share_migration_with_different_snapshot_capability_type(
            self, force_host_assisted, snapshot_capable):

        self._check_migration_enabled(force_host_assisted)
        ss_type, no_ss_type = self._create_share_type_for_snapshot_capability()

        if snapshot_capable:
            share_type = ss_type
            share_type_id = no_ss_type['id']
            new_share_type_id = ss_type['id']
        else:
            share_type = no_ss_type
            share_type_id = ss_type['id']
            new_share_type_id = no_ss_type['id']

        share = self.create_share(
            self.protocol, share_type_id=share_type_id)
        share = self.shares_v2_client.get_share(share['id'])['share']

        if snapshot_capable:
            self.assertEqual(False, share['snapshot_support'])
        else:
            # Verify that share has snapshot support capability
            self.assertTrue(share['snapshot_support'])

        dest_pool = utils.choose_matching_backend(share, self.pools,
                                                  share_type)
        task_state, new_share_network_id, __ = (
            self._get_migration_data(share, force_host_assisted))
        share = self.migrate_share(
            share['id'], dest_pool['name'],
            force_host_assisted_migration=force_host_assisted,
            wait_for_status=task_state,
            new_share_type_id=new_share_type_id,
            new_share_network_id=new_share_network_id)
        share = self.migration_complete(share['id'], dest_pool)

        if snapshot_capable:
            # Verify that migrated share does have snapshot support capability
            self.assertTrue(share['snapshot_support'])
        else:
            # Verify that migrated share don't have snapshot support capability
            self.assertEqual(False, share['snapshot_support'])

        self._cleanup_share(share)

    def _create_share_type_for_snapshot_capability(self):
        # Share type with snapshot support
        st_name = data_utils.rand_name(
            'snapshot_capable_share_type_for_migration')
        extra_specs = self.add_extra_specs_to_dict({
            "snapshot_support": True,
            "create_share_from_snapshot_support": True,
        })
        ss_type = self.create_share_type(st_name, extra_specs=extra_specs)

        # New share type with no snapshot support capability
        # to which a share will be migrated
        new_st_name = data_utils.rand_name(
            'snapshot_noncapable_share_type_for_migration')
        extra_specs = {
            "driver_handles_share_servers": CONF.share.multitenancy_enabled
        }
        no_ss_type = self.create_share_type(new_st_name,
                                            extra_specs=extra_specs)
        return ss_type, no_ss_type

    def _cleanup_share(self, share):
        resource = {"type": "share", "id": share["id"],
                    "client": self.shares_v2_client}
        # NOTE(Yogi1): Share needs to be cleaned up explicitly at the end of
        #  test otherwise, newly created share_network will not get cleaned up.
        self.method_resources.insert(0, resource)


@ddt.ddt
class MigrationCancelNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('d39dfa1b-6e91-4efc-84f1-76f878b51f2a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @ddt.data(True, False)
    def test_migration_cancel(self, force_host_assisted):
        self._check_migration_enabled(force_host_assisted)

        share = self.create_share(self.protocol,
                                  share_type_id=self.share_type_id)
        share = self.shares_v2_client.get_share(share['id'])['share']
        share, dest_pool = self._setup_migration(share)
        task_state = (constants.TASK_STATE_DATA_COPYING_COMPLETED
                      if force_host_assisted
                      else constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE)

        share = self.migrate_share(
            share['id'], dest_pool, wait_for_status=task_state,
            force_host_assisted_migration=force_host_assisted)

        self._validate_migration_successful(
            dest_pool, share, task_state, complete=False)

        progress = self.shares_v2_client.migration_get_progress(share['id'])

        self.assertEqual(task_state, progress['task_state'])
        self.assertEqual(100, progress['total_progress'])

        share = self.migration_cancel(share['id'], dest_pool)
        progress = self.shares_v2_client.migration_get_progress(share['id'])

        self.assertEqual(
            constants.TASK_STATE_MIGRATION_CANCELLED, progress['task_state'])
        self.assertEqual(100, progress['total_progress'])

        self._validate_migration_successful(
            dest_pool, share, constants.TASK_STATE_MIGRATION_CANCELLED,
            complete=False)

    @decorators.idempotent_id('640dce56-2084-488d-8dea-456840ff397e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(
        CONF.share.run_snapshot_tests, 'Snapshot tests are disabled.')
    @testtools.skipUnless(
        CONF.share.run_driver_assisted_migration_tests,
        'Driver-assisted migration tests are disabled.')
    @testtools.skipUnless(
        CONF.share.run_migration_with_preserve_snapshots_tests,
        'Migration with preserve snapshots tests are disabled.')
    def test_migration_cancel_share_with_snapshot(self):
        share = self.create_share(self.protocol,
                                  share_type_id=self.share_type_id)
        share = self.shares_v2_client.get_share(share['id'])['share']

        share, dest_pool = self._setup_migration(share)
        snapshot1 = self.create_snapshot_wait_for_active(share['id'])
        snapshot2 = self.create_snapshot_wait_for_active(share['id'])

        task_state, new_share_network_id, new_share_type_id = (
            self._get_migration_data(share))

        share = self.migrate_share(
            share['id'], dest_pool,
            wait_for_status=task_state, new_share_type_id=new_share_type_id,
            new_share_network_id=new_share_network_id, preserve_snapshots=True)

        share = self.migration_cancel(share['id'], dest_pool)
        self._validate_snapshot(share, snapshot1, snapshot2)


@ddt.ddt
class MigrationOppositeDriverModesNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('d8cce50d-e8da-4fbc-8f94-0827bf277b6c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @ddt.data(True, False)
    def test_migration_opposite_driver_modes(self, force_host_assisted):
        self._check_migration_enabled(force_host_assisted)

        if not CONF.share.multitenancy_enabled:
            # If currently configured is DHSS=False,
            # then we need it for DHSS=True
            new_share_network_id = self.provide_share_network(
                self.shares_v2_client,
                self.networks_client,
                ignore_multitenancy_config=True,
            )
        else:
            # If currently configured is DHSS=True,
            # then we must pass None for DHSS=False
            new_share_network_id = None

        share = self.create_share(self.protocol,
                                  share_type_id=self.share_type_id,
                                  cleanup_in_class=False)
        share = self.shares_v2_client.get_share(share['id'])['share']
        share, dest_pool = self._setup_migration(share, opposite=True)

        old_share_network_id = share['share_network_id']
        old_share_type_id = share['share_type']
        new_share_type_id = self.new_type_opposite['id']

        task_state = (constants.TASK_STATE_DATA_COPYING_COMPLETED
                      if force_host_assisted
                      else constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE)

        share = self.migrate_share(
            share['id'], dest_pool,
            force_host_assisted_migration=force_host_assisted,
            wait_for_status=task_state, new_share_type_id=new_share_type_id,
            new_share_network_id=new_share_network_id)

        self._validate_migration_successful(
            dest_pool, share, task_state, complete=False,
            share_network_id=old_share_network_id,
            share_type_id=old_share_type_id)

        progress = self.shares_v2_client.migration_get_progress(share['id'])

        self.assertEqual(task_state, progress['task_state'])
        self.assertEqual(100, progress['total_progress'])

        share = self.migration_complete(share['id'], dest_pool)

        progress = self.shares_v2_client.migration_get_progress(share['id'])

        self.assertEqual(
            constants.TASK_STATE_MIGRATION_SUCCESS, progress['task_state'])
        self.assertEqual(100, progress['total_progress'])

        self._validate_migration_successful(
            dest_pool, share, constants.TASK_STATE_MIGRATION_SUCCESS,
            complete=True, share_network_id=new_share_network_id,
            share_type_id=new_share_type_id)


@ddt.ddt
class MigrationTwoPhaseNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('e6cf0e4d-bdf3-49c1-b6ba-56d1ad6c81d2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @ddt.data(True, False)
    def test_migration_2phase(self, force_host_assisted):
        self._check_migration_enabled(force_host_assisted)

        share = self.create_share(self.protocol,
                                  share_type_id=self.share_type_id)
        share = self.shares_v2_client.get_share(share['id'])['share']
        share, dest_pool = self._setup_migration(share)

        old_share_network_id = share['share_network_id']
        old_share_type_id = share['share_type']
        task_state, new_share_network_id, new_share_type_id = (
            self._get_migration_data(share, force_host_assisted))

        share = self.migrate_share(
            share['id'], dest_pool,
            force_host_assisted_migration=force_host_assisted,
            wait_for_status=task_state, new_share_type_id=new_share_type_id,
            new_share_network_id=new_share_network_id)

        self._validate_migration_successful(
            dest_pool, share, task_state, complete=False,
            share_network_id=old_share_network_id,
            share_type_id=old_share_type_id)

        progress = self.shares_v2_client.migration_get_progress(share['id'])

        self.assertEqual(task_state, progress['task_state'])
        self.assertEqual(100, progress['total_progress'])

        share = self.migration_complete(share['id'], dest_pool)

        progress = self.shares_v2_client.migration_get_progress(share['id'])

        self.assertEqual(
            constants.TASK_STATE_MIGRATION_SUCCESS, progress['task_state'])
        self.assertEqual(100, progress['total_progress'])

        self._validate_migration_successful(
            dest_pool, share, constants.TASK_STATE_MIGRATION_SUCCESS,
            complete=True, share_network_id=new_share_network_id,
            share_type_id=new_share_type_id)
        self._cleanup_share(share)


@ddt.ddt
class MigrationWithShareExtendingNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('0e3d75e0-385a-4f7a-889f-2a3db79db8c2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(
        CONF.share.run_extend_tests, 'Extend share tests are disabled.')
    @ddt.data(True, False)
    def test_extend_on_migrated_share(self, force_host_assisted):
        self._test_resize_post_migration(force_host_assisted, resize='extend')


@ddt.ddt
class MigrationWithShareShrinkingNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('58c72e51-d217-48bc-8155-5a010912312e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(
        CONF.share.run_shrink_tests, 'Shrink share tests are disabled.')
    @ddt.data(True, False)
    def test_shrink_on_migrated_share(self, force_host_assisted):
        self._test_resize_post_migration(force_host_assisted, resize='shrink')


@ddt.ddt
class MigrationOfShareWithSnapshotNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('a95eb701-626a-4175-967b-4880d3716857')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(
        CONF.share.run_snapshot_tests, 'Snapshot tests are disabled.')
    @testtools.skipUnless(
        CONF.share.run_driver_assisted_migration_tests,
        'Driver-assisted migration tests are disabled.')
    @testtools.skipUnless(
        CONF.share.run_migration_with_preserve_snapshots_tests,
        'Migration with preserve snapshots tests are disabled.')
    def test_migrating_share_with_snapshot(self):
        ss_type, __ = self._create_share_type_for_snapshot_capability()

        share = self.create_share(self.protocol,
                                  share_type_id=ss_type['id'],
                                  cleanup_in_class=False)
        share = self.shares_v2_client.get_share(share['id'])['share']

        share, dest_pool = self._setup_migration(share)
        snapshot1 = self.create_snapshot_wait_for_active(
            share['id'], cleanup_in_class=False)
        snapshot2 = self.create_snapshot_wait_for_active(
            share['id'], cleanup_in_class=False)

        task_state, new_share_network_id, __ = self._get_migration_data(share)

        share = self.migrate_share(
            share['id'], dest_pool,
            wait_for_status=task_state,
            new_share_type_id=ss_type['id'],
            new_share_network_id=new_share_network_id, preserve_snapshots=True)

        share = self.migration_complete(share['id'], dest_pool)

        self._validate_snapshot(share, snapshot1, snapshot2)


@ddt.ddt
class MigrationWithDifferentSnapshotSupportNFSTest(MigrationBase):
    protocol = "nfs"

    @decorators.idempotent_id('a18b3637-2070-4a1b-acd9-c392eb7963b5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          'Snapshot tests are disabled.')
    @ddt.data(True, False)
    def test_migrate_share_to_snapshot_capability_share_type(
            self, force_host_assisted):
        # Verify that share with no snapshot support type can be migrated
        # to new share type which supports the snapshot
        self._validate_share_migration_with_different_snapshot_capability_type(
            force_host_assisted, True)

    @decorators.idempotent_id('59313673-6576-4163-ab96-41bafcdad63a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          'Snapshot tests are disabled.')
    @ddt.data(True, False)
    def test_migrate_share_to_no_snapshot_capability_share_type(
            self, force_host_assisted):
        # Verify that share with snapshot support type can be migrated
        # to new share type which doesn't support the snapshot
        self._validate_share_migration_with_different_snapshot_capability_type(
            force_host_assisted, False)


# NOTE(u_glide): this function is required to exclude MigrationBase from
# executed test cases.
# See: https://docs.python.org/3/library/unittest.html#load-tests-protocol
# for details.
def load_tests(loader, tests, _):
    result = []
    for test_case in tests:
        if not test_case._tests or type(test_case._tests[0]) is MigrationBase:
            continue
        result.append(test_case)
    return loader.suiteClass(result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ReplicationExportLocationsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationExportLocationsNegativeTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')

    @classmethod
    def resource_setup(cls):
        super(ReplicationExportLocationsNegativeTest, cls).resource_setup()
        # Create share_type
        name = data_utils.rand_name(constants.TEMPEST_MANILA_PREFIX)
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )
        cls.extra_specs = cls.add_extra_specs_to_dict(
            {"replication_type": cls.replication_type})
        cls.share_type = cls.create_share_type(
            name,
            extra_specs=cls.extra_specs,
            client=cls.admin_client)
        cls.sn_id = None
        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.sn_id = cls.share_network['id']
        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

    @decorators.idempotent_id('7a6318c2-71e9-45d2-aafe-5dcc0810a14a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.47')
    @testtools.skipUnless(
        CONF.share.backend_replication_type in
        (constants.REPLICATION_STYLE_READABLE, constants.REPLICATION_STYLE_DR),
        'Test is not appropriate for writable replication style.')
    def test_get_share_export_location_for_secondary_replica(self):
        """Is NotFound raised with share el API for non-active replicas"""
        share = self.create_share(share_type_id=self.share_type['id'],
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        replica = self.create_share_replica(share['id'], self.replica_zone)
        replica_exports = (
            self.shares_v2_client.list_share_replica_export_locations(
                replica['id'])
        )['export_locations']

        for export in replica_exports:
            self.assertRaises(lib_exc.NotFound,
                              self.shares_v2_client.get_share_export_location,
                              share['id'], export['id'])

    @decorators.idempotent_id('f1b89548-4da7-4ce1-a3c0-363a6c1ea726')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.47')
    def test_get_replica_export_location_for_non_replica(self):
        """Is NotFound raised for non-replica share instances"""
        # Create a share type with no support for replication
        share_type = self.create_share_type()
        share = self.create_share(share_type_id=share_type['id'],
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        share_instances = self.admin_client.get_instances_of_share(
            share['id'])['share_instances']
        for instance in share_instances:
            self.assertRaises(
                lib_exc.NotFound,
                self.shares_v2_client.list_share_replica_export_locations,
                instance['id'])

    @decorators.idempotent_id('f60d7eec-ec29-4767-985f-ff8c9e771cae')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported('2.47')
    def test_list_replica_export_locations_for_invalid_replica(self):
        """Is NotFound raised for invalid replica ID"""
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.list_share_replica_export_locations,
            'invalid-replica-id')

    @decorators.idempotent_id('29ec890e-e324-4487-b2e3-f87642b6a58d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.47')
    def test_get_replica_export_location_for_invalid_export_id(self):
        """Is NotFound raised for invalid replica export location ID"""
        share = self.create_share(share_type_id=self.share_type['id'],
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        replica = self.create_share_replica(share['id'], self.replica_zone)
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.get_share_replica_export_location,
            replica['id'], 'invalid-export-location-id')



# Copyright 2014 mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


class ExtensionsTest(base.BaseSharesTest):

    @decorators.idempotent_id('befd2105-c9f6-4eca-a693-9900eb0fb37a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_extensions(self):

        # get extensions
        extensions = self.shares_client.list_extensions()['extensions']

        # verify response
        keys = ["alias", "updated", "name", "description"]
        [self.assertIn(key, ext.keys()) for ext in extensions for key in keys]
